/*
 * transport.h
 *
 * Function declarations and structure definitions for transport.c
 *
 * Copyright (c) 2004, 2005 Todd MacDermid <tmacd@synacklabs.net>
 *
 */

#ifndef TRANSPORT_H
#define TRANSPORT_H

/*
 * The cutlass transport init header contains information about what the
 * transport will be transporting
 */

struct cutlass_transport_init_hdr {
  uint8_t type;
  uint16_t init_hdr_len;
  uint32_t request_id;
  uint32_t transport_size;
};

#define TRANSPORT_INIT_HDR_LEN_MIN 11

/*
 * The cutlass transport send packet contains the offset that the
 * data resides at in the header, and then the data in the body.
 */

struct cutlass_transport_send_hdr {
  uint32_t offset;
};

#define CUT_TRANS_SEND_HDR_LEN 4

/*
 * The cutlass transport acknowledgement header lists the number of gaps 
 * still contained within cutlass, and then the gaps themselves, in network
 * byte-ordered uint32s
 */

struct cutlass_transport_ack_hdr {
  uint16_t num_gaps;
};

#define CUT_TRANSPORT_ACK_HDR_LEN 2

/* Function declarations beloooooooow */

poll_t *
cutlass_transport_poll(cutlass_t *cut_handle, uint8_t *fingerprint,
                       int channel_id);

int
cutlass_transport_init_send(cutlass_t *cut_handle, conn_t *conn,
			    struct channel *transport_info,
			    struct timespec *now);

int
cutlass_transport_init_parse(cutlass_t *cut_handle, conn_t *conn, 
			     uint8_t *clear_packet, 
			     struct cutlass_packet_hdr *packet_info,
			     struct timespec *now);

int
cutlass_transport_init_ack_send(cutlass_t *cut_handle, conn_t *conn,
				struct channel *transport_info, 
				uint8_t *clear_packet,
				struct timespec *now);

int
cutlass_transport_init_ack_parse(cutlass_t *cut_handle, conn_t *conn, 
				 uint8_t *clear_packet, 
				 struct cutlass_packet_hdr *packet_info, 
				 struct timespec *now);

int
cutlass_transport_send(cutlass_t *cut_handle, conn_t *conn,
		       struct channel *transport_info, 
		       struct timespec *now);

int
cutlass_transport_parse(cutlass_t *cut_handle, conn_t *conn, 
			uint8_t *clear_packet, 
			struct cutlass_packet_hdr *packet_info, 
			struct timespec *now);

int
cutlass_transport_ack_send(cutlass_t *cut_handle, conn_t *conn, 
			   struct channel *transport_info, 
			   uint8_t *clear_packet, struct timespec *now);

int
cutlass_transport_ack_parse(cutlass_t *cut_handle, conn_t *conn, 
			    uint8_t *clear_packet, 
			    struct cutlass_packet_hdr *packet_info, 
			    struct timespec *now);

int
cutlass_transport_housekeeping(cutlass_t *cut_handle, conn_t *conn,
			       struct channel *transport_info, int direction,
			       struct timespec *now, 
			       struct timespec *sleep_time);

int
cutlass_transport_complete(cutlass_t *cut_handle, conn_t *conn,
			   struct channel *transport_info);

int
cutlass_file_send_complete(cutlass_t *cut_handle, conn_t *conn,
			   struct channel *transport_info);


#endif /* TRANSPORT_H */
